@@ -2,8 +2,8 @@
 #
 # Copyright (C) 2002  Masato Taruishi <taru@debian.org>
 # Copyright (C) 2006-2008  Junichi Uekawa <dancer@debian.org>
-# Copyright (C) 2008-2010  Francesco Poli <frx@firenze.linux.it>
-# Copyright (C) 2009  Ryan Niebur <ryan@debian.org>
+# Copyright (C) 2008-2011  Francesco Poli <invernomuto@paranoici.org>
+# Copyright (C) 2009-2010  Ryan Niebur <ryan@debian.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -42,15 +42,15 @@ class AppConfig
       "\n",
       _("Options:\n"),
       _(" -h               : Display this help and exit.\n"),
-      _(" -v               : Show version number and exit\n"),
+      _(" -v               : Show version number and exit.\n"),
       sprintf(_(" -s <severities>  : Severities you want to see [%s], or [all].\n"), @severity.join(',')),
       _(" -T <tags>        : Tags you want to see.\n"),
       sprintf(_(" -S <stats>       : Stats you want to see [%s].\n"), @stats.join(',')),
       _(" -B <bug#>        : Restrict reporting to specified bug#s only.\n"),
       _(" -D               : Show downgraded packages, too.\n"),
       sprintf(_(" -H <hostname>    : Hostname of Debian Bug Tracking System [%s].\n"), @hostname),
-      sprintf(_(" -p <port>        : Port number of the server [%s]\n"), @port),
-      sprintf(_(" -P <priority>    : Specifies Pin-Priority value [%s]\n"), @pin_priority),
+      sprintf(_(" -p <port>        : Port number of the server [%s].\n"), @port),
+      sprintf(_(" -P <priority>    : Specifies Pin-Priority value [%s].\n"), @pin_priority),
       _(" -E <title>       : Specifies the title of rss output.\n"),
       _(" -q               : Don't display progress bar.\n"),
       _(" -C <apt.conf>    : Specify apt.conf.\n"),
@@ -58,9 +58,9 @@ class AppConfig
       _(" -n               : Assume that you select no for all questions.\n"),
       _(" -d               : Debug.\n"),
       _("Commands:\n"),
-      _(" apt              : apt mode\n"),
-      _(" list <pkg...>    : list bug reports of the specified packages\n"),
-      _(" rss <pkg...>     : list bug reports of the specified packages in rss\n"),
+      _(" apt              : Apt mode.\n"),
+      _(" list <pkg...>    : List bug reports of the specified packages.\n"),
+      _(" rss <pkg...>     : List bug reports of the specified packages in rss.\n"),
       _("See the manual page for the long options.\n")
   end
 
@@ -68,7 +68,7 @@ class AppConfig
     @severity = ["critical", "grave", "serious"]
     @tag = nil
     @stats = ["forwarded", "done", "pending", "pending-fixed", ""]
-    @statmap = [["forwarded", _("forwarded")], 
+    @statmap = [["forwarded", _("forwarded")],
                 ["done", _("marked as done in some version")],
                 ["pending", _("unfixed")],
                 ["pending-fixed", _("tagged as pending a fix")]]
@@ -118,7 +118,7 @@ class AppConfig
 			   ['--force-no', '-n', GetoptLong::NO_ARGUMENT],
 			   ['--debug', '-d', GetoptLong::NO_ARGUMENT]
 			   );
-    
+
     begin
       opt_parser.each_option { |optname, optargs|
 	case optname
@@ -173,42 +173,44 @@ class AppConfig
       @quiet = true
       @yes = false if @yes.nil?
     end
- 
-    @title = "Debian Bugs of #{Socket.gethostname} (#{@severity.join(', ')})" if ! @title
+
+    @title = "Debian Bugs (#{@severity.join(', ')})" if ! @title
 
     # http_proxy sanity check
     if ENV["HTTP_PROXY"] != nil && ENV["http_proxy"] == nil
-      $stderr.print _("W: sanity check failed: environment variable http_proxy is unset and HTTP_PROXY is set.")
+      $stderr.puts _("W: sanity check failed: environment variable http_proxy is unset and HTTP_PROXY is set.")
     end
+
+    # enable proxy for SOAP
     if ENV["http_proxy"] != nil && ENV["soap_use_proxy"] != "on"
-      $stderr.print _("W: sanity check failed: environment variable http_proxy is set and soap_use_proxy is not 'on'.")
+      ENV["soap_use_proxy"] = "on"
     end
 
-    # http_proxy check
-      if /http_proxy='(.*)'/ =~ `apt-config #{@apt_conf} shell http_proxy acquire::http::proxy` 
-        puts "proxy configuration from APT.CONF: #{$1}" if $DEBUG
-        if $1 == 'DIRECT' || $1 == ''
-          puts "Disabling proxy due to DIRECT, or empty string" if $DEBUG          
-          ENV.delete("http_proxy")
-          ENV.delete("soap_use_proxy")
-        else
-          ENV["http_proxy"] = $1
-          ENV["soap_use_proxy"] = "on"
-        end
+    # proxy settings in apt.conf
+    if /http_proxy='(.*)'/ =~ `apt-config #{@apt_conf} shell http_proxy acquire::http::proxy`
+      puts "proxy configuration from apt.conf: #{$1}" if $DEBUG
+      if $1 == 'DIRECT' || $1 == ''
+        puts "Disabling proxy due to DIRECT, or empty string" if $DEBUG
+        ENV.delete("http_proxy")
+        ENV.delete("soap_use_proxy")
+      else
+        ENV["http_proxy"] = $1
+        ENV["soap_use_proxy"] = "on"
       end
-      if /http_proxy='(.*)'/ =~ `apt-config #{@apt_conf} shell http_proxy acquire::http::proxy::bugs.debian.org` 
-        if $1 == 'DIRECT' 
-          puts "Disabling proxy due to DIRECT" if $DEBUG
-          ENV.delete("http_proxy")
-          ENV.delete("soap_use_proxy")
-        else
-          puts "proxy configuration from APT.CONF, specific for bugs.debian.org: #{$1}" if $DEBUG
-          ENV["http_proxy"] = $1
-          ENV["soap_use_proxy"] = "on"
-        end
+    end
+    if /http_proxy='(.*)'/ =~ `apt-config #{@apt_conf} shell http_proxy acquire::http::proxy::bugs.debian.org`
+      puts "proxy configuration from apt.conf, specific for bugs.debian.org: #{$1}" if $DEBUG
+      if $1 == 'DIRECT'
+        puts "Disabling proxy due to DIRECT" if $DEBUG
+        ENV.delete("http_proxy")
+        ENV.delete("soap_use_proxy")
+      else
+        ENV["http_proxy"] = $1
+        ENV["soap_use_proxy"] = "on"
       end
+    end
 
-    # command 
+    # command
     command = ARGV.shift
     case command
     when nil
@@ -228,8 +230,8 @@ class AppConfig
     end
 
     @parser =
-      Debian::BTS::Parser::SoapIndex.new(@hostname, @port)	    
-    
+      Debian::BTS::Parser::SoapIndex.new(@hostname, @port)
+
     if FileTest.executable?("#{QUERYBTS}")
       @querybts = QUERYBTS
     end
@@ -286,7 +288,7 @@ class IgnoreBugs < Array
   def add(entry, write=true)
     if write == true
       @@path_mutex[@path].synchronize {
-        begin 
+        begin
           open(@path, "a") { |file|
             file.puts entry
           }
@@ -330,12 +332,12 @@ class Viewer
 	ask_str = _("Are you sure you want to install/upgrade the above packages?").dup
 	if @config.querybts != nil || @config.browser != nil
 	  if hold_pkgs.empty?
-	    ask_str << " [Y/n/?/...] "
+	    ask_str << " [Y/n/?/...]"
 	  else
-	    ask_str << " [N/?/...] "
+	    ask_str << " [N/?/...]"
 	  end
 	else
-	  ask_str << " [Y/n] "
+	  ask_str << " [Y/n]"
 	end
 	if @config.yes.nil?
           a = @config.frontend.ask ask_str
@@ -424,7 +426,7 @@ class Viewer
             }
 	  end
 	  if pkgs.size != 0
-	    if @config.frontend.yes_or_no? sprintf(ngettext("The following %d package will be pinned or on hold:\n %s\nAre you sure ", "The following %d packages will be pinned or on hold:\n %s\nAre you sure ", pkgs.size), pkgs.size, pkgs.keys.join(', '))
+	    if @config.frontend.yes_or_no? sprintf(ngettext("The following %d package will be pinned or on hold:\n %s\nAre you sure?", "The following %d packages will be pinned or on hold:\n %s\nAre you sure?", pkgs.size), pkgs.size, pkgs.keys.join(', '))
 	      if key == "h"
                 h = on_hold(pkgs.keys)
               else
@@ -585,7 +587,7 @@ Pin-Priority: #{pin_pri}
       displayed_pkgs = []
 
       tmp = HtmlTempfile.new("apt-listbugs")
-      tmp.puts "<html><head><title>"+_("critical bugs for your upgrade")+"</title><meta http-equiv=\"Content-Type\" content=\"text/html; charset=#{Locale.codeset}\"></head><body>"
+      tmp.puts "<html><head><title>"+_("critical bugs for your upgrade")+"</title><meta http-equiv=\"Content-Type\" content=\"text/html; charset=#{Locale.charset}\"></head><body>"
       tmp.puts _("<h1 align=\"center\">Critical bugs for your upgrade</h1>")
       tmp.puts _("<p align=\"right\">by apt-listbugs</p><hr>")
       tmp.puts _("<h2>Bug reports</h2>")
@@ -610,17 +612,38 @@ Pin-Priority: #{pin_pri}
       tmp.puts "</body></html>"
       tmp.close
 
-      puts "Invoking www-browswer for #{tmp.path}" if $DEBUG
+      puts "Invoking browser for #{tmp.path}" if $DEBUG
       browsercommandline = ""
       browsercommandline << "su #{ENV["SUDO_USER"]} -c \"" if ENV["SUDO_USER"]
       browsercommandline << "#{@config.browser} #{tmp.path} < /dev/tty"
       browsercommandline << "\"" if ENV["SUDO_USER"]
       if system(browsercommandline)
-        puts "successfully invoked www-browswer" if $DEBUG
+        puts "successfully invoked browser" if $DEBUG
       else
-        $stderr.puts _("W: Failed to invoke www-browser.")
+        $stderr.puts _("W: Failed to invoke browser.")
         $stderr.puts " #{browsercommandline}"
       end
+      clear_stdin
+    end
+
+    private
+    def clear_stdin(parent = true)
+      fd = @config.frontend.tty
+      flags=fd.fcntl(Fcntl::F_GETFL)
+      if parent
+        while clear_stdin(false)
+          nil
+        end
+      else
+        begin
+          fd.read_nonblock(10000000)
+          return true
+        rescue Errno::EAGAIN
+          return false
+        ensure
+          fd.fcntl(Fcntl::F_SETFL, flags)
+        end
+      end
     end
   end
 
@@ -677,7 +700,7 @@ Pin-Priority: #{pin_pri}
       }
       @config.frontend.puts rss.to_s
     end
-    
+
   end
 
 end
@@ -706,120 +729,15 @@ module Factory
   module_function :config, :config=, :create, :done?
   public :create
 
-  module PackageFactory
+  module BugsFactory
     extend Factory
 
-    # parse 'apt-listbugs list pkgname/version .... ' combination.
-    module ListFactory
-      include Factory
-      def field(pkgname)
-	f = {}
-        case pkgname
-        when /^(.*)\/(.*)$/
-          f["package"] = $1
-          f["version"] = $2
-        else
-          f["package"] = pkgname
-        end
-	f
-      end
-      module_function :field
-    end
-
-    module AptFactory
-      include Factory
-      DPKG = "/usr/bin/dpkg"
-      def field(pkgpath)
-        return Debian::Dpkg.field(pkgpath)
-      end
-      module_function :field
-    end
-
-    def create(pkgnames, *args)
-      # obtain package names and get information about them
-      step = 100.0 / pkgnames.size.to_f
-      pkgs = {}
-      reading = _("Reading package fields...")
-      pkgnames.each_index { |index|
-	if block_given?
-	  yield reading, (index * step).to_i.to_s + "%"
-	end
-	pkgnames[index].chomp!
-	f = nil
-	case config.command
-	when "apt"
-	  f = AptFactory.field(pkgnames[index])
-	when "list"
-	  f = ListFactory.field(pkgnames[index])
-	when "rss"
-	  f = ListFactory.field(pkgnames[index])
-	else
-	  raise _("Not Implemented")
-	end
-	pkgs[f["package"]] = f
-      }
-      if block_given?
-	yield reading, _("Done")
-      end
-      pkgs
-    end
-    
     def delete_ignore_pkgs(new_pkgs)
       new_pkgs.delete_if { |name, pkg|
 	config.system_ignore_bugs.include?(name)
       }
     end
 
-    module_function :create, :delete_ignore_pkgs
-
-  end
-
-  module StatusFactory
-    extend Factory
-
-    def create(pkgs, *args)
-      # creating status database, which contains the list of packages.
-
-      if block_given?
-	yield _("Reading package status..."), "0%"
-      end
-      pkgres = []
-      pkgs.each_key { |pkg|
-        pkgres << "#{pkg}"
-      }      
-      i=0
-      max=pkgres.size
-      step=(pkgres.size/100)*10+1
-      status = Debian::Packages.new(Debian::Dpkg::STATUS_FILE, pkgres) do
-	yield _("Reading package status..."), 
-          "#{(i.to_f/max.to_f*100).to_i}%" if (i % step) == 0
-	i += 1
-      end
-      if block_given?
-	yield _("Reading package status..."), _("Done")
-      end
-      status
-    end
-
-    def delete_downgraded(cur_pkgs, new_pkgs)
-      new_pkgs.delete_if { |name, pkg|
-	val = false
-	ver = pkg["version"]
-	cur_ver = cur_pkgs[name]["version"] if cur_pkgs[name] != nil
-	if ver != nil && cur_ver != nil
-	  val = true if Debian::Dpkg.compare_versions(ver, "lt", cur_ver)
-	end
-	val
-      }
-    end
-
-    module_function :create, :delete_downgraded
-
-  end
-
-  module BugsFactory
-    extend Factory
-
     def create(new_pkgs, *args, &progress)
       cur_pkgs = args[0]
       bugs = Debian::Bugs.new
@@ -834,7 +752,7 @@ module Factory
         # obtain a list of package names
         tmppkgs = []
         new_pkgs.each_key { |k| tmppkgs << k }
-        
+
         # send the list of package names and severity to be parsed.
         bugs = config.parser.parse(tmppkgs, config.severity) { |pct|
           yield _("Retrieving bug reports..."), pct
@@ -845,7 +763,7 @@ module Factory
         $stderr.puts _(" E: HTTP GET failed")
         retrycount -= 1
         retry if config.frontend.yes_or_no?(_("Retry downloading bug information?")) && retrycount > 0
-        raise _("Exiting with error") if config.frontend.yes_or_no?(_("Abort the installation"))
+        raise _("Exiting with error") if ! config.frontend.yes_or_no?(_("Continue the installation anyway?"), false)
         bugs = []
       rescue SOAP::EmptyResponseError => exception
         config.frontend.puts _(" Fail")
@@ -853,7 +771,7 @@ module Factory
         $stderr.puts _(" E: Empty stream from SOAP")
         retrycount -= 1
         retry if config.frontend.yes_or_no?(_("Retry downloading bug information?")) && retrycount > 0
-        raise _("Exiting with error") if config.frontend.yes_or_no?(_("Abort the installation"))
+        raise _("Exiting with error") if ! config.frontend.yes_or_no?(_("Continue the installation anyway?"), false)
         bugs = []
       rescue Exception => exception
         config.frontend.puts _(" Fail")
@@ -861,17 +779,17 @@ module Factory
         config.frontend.puts _("Error retrieving bug reports from the server with the following error message:")
         config.frontend.puts " W: #{$!}"
         if exception.kind_of? SocketError
-          config.frontend.puts _("It appears that your network connection is down. Check network configuration and try again") 
+          config.frontend.puts _("It appears that your network connection is down. Check network configuration and try again")
         else
-          config.frontend.puts _("It could be because your network is down, or because of broken proxy servers, or the BTS server itself is down. Check network configuration and try again") 
+          config.frontend.puts _("It could be because your network is down, or because of broken proxy servers, or the BTS server itself is down. Check network configuration and try again")
         end
         retrycount -= 1
         retry if config.frontend.yes_or_no?(_("Retry downloading bug information?")) && retrycount > 0
-        raise _("Exiting with error") if config.frontend.yes_or_no?(_("Abort the installation"))
+        raise _("Exiting with error") if ! config.frontend.yes_or_no?(_("Continue the installation anyway?"), false)
         bugs = []
       end
       yield _("Retrieving bug reports..."), "100%"
-      
+
       if block_given?
         yield _("Retrieving bug reports..."), _("Done")
       end
@@ -913,14 +831,14 @@ module Factory
           yield version
         end
       }
-    end      
+    end
 
     def find_max_version_below_ver(space_delimited_bts_version, new_ver, pkg_name)
       # find the max version from found/fixed that is below new_ver
       # data format of space_delimited_bts_version:
       # space-delimited PACKAGE/VERSION, or VERSION.
 
-      maxver=nil 
+      maxver=nil
       iterate_fixed_found_version(space_delimited_bts_version, pkg_name) { |each_ver|
         # check each fixed_ver
         if Debian::Dpkg.compare_versions(each_ver, "le", new_ver) &&
@@ -930,13 +848,13 @@ module Factory
       }
       maxver
     end
-    
+
     def find_min_version_above_ver(space_delimited_bts_version, new_ver, pkg_name)
       # find the min version from found/fixed that is above new_ver
       # data format of space_delimited_bts_version:
       # space-delimited PACKAGE/VERSION, or VERSION.
 
-      minver=nil 
+      minver=nil
       iterate_fixed_found_version(space_delimited_bts_version, pkg_name) { |each_ver|
         # check each each_ver
         if Debian::Dpkg.compare_versions(each_ver, "ge", new_ver) &&
@@ -946,19 +864,19 @@ module Factory
       }
       minver
     end
-    
+
     def am_i_buggy(ver, fixed, found)
       # find out if this version is buggy or not depending on the fixed / found array.
       # how do I handle casees of found / fixed = nil?
 
       puts " .. checking ver #{ver} against fixed:#{fixed} found:#{found}" if $DEBUG
-      
+
       fixed_max_below = nil
       found_max_below = nil
 
       fixed_min_above = nil
       found_min_above = nil
-      
+
       fixed_max_below = find_max_version_below_ver(fixed, ver, name) if ! fixed.nil?
       found_max_below = find_max_version_below_ver(found, ver, name) if ! found.nil?
 
@@ -970,11 +888,11 @@ module Factory
       # if ( found_max_below <= fixed_max_below ), I know it's fixed by ver.
       # there are cases where found_max_below and fixed_max_below isn't available, ingnore it.
 
-      if fixed_max_below == nil && found_max_below == nil 
+      if fixed_max_below == nil && found_max_below == nil
         # this is a hard thing to handle, but do some guessing here:
-        # bug was not fixed or found before this version in consideration. 
+        # bug was not fixed or found before this version in consideration.
         # It either means it wasn't found before this version, or 'found' version info is missing from BTS.
-        if found_min_above == nil 
+        if found_min_above == nil
           # no new version found; which I guess signifies that the BTS is missing version info.
           puts " ... found is nil; I guess I am buggy" if $DEBUG
           val=true
@@ -983,7 +901,7 @@ module Factory
           # buggy; except for a case where it's fixed before the
           # found_min, which probably means 'found' info is missing
           # from BTS.
-          if fixed_min_above == nil || 
+          if fixed_min_above == nil ||
               Debian::Dpkg.compare_versions(fixed_min_above, "gt", found_min_above)
             puts " ... Bug is 'found' on a later version, I guess I am not buggy" if $DEBUG
             val=false
@@ -1002,52 +920,38 @@ module Factory
                 else
                   # default path.
                 end
-           end 
+           end
       end
 
       val
     end
-    
+
     def bug_is_irrelevant(name, cur_ver, new_ver, bug_number, fixed, found, bug_stat="")
       # find out if the bug number is irrelevant for this specific upgrade, from fixed/found information.
       # @return false: bug is relevant, true: bug is irrelevant, should be removed.
-      val=false
+      val = false
 
       puts "Start checking: #{bug_number}" if $DEBUG
 
-      if config.command == "list"
-        puts "Special handling list for cur_ver:#{cur_ver} and new_ver:#{new_ver}" if $DEBUG
-        new_ver=cur_ver if new_ver.nil?
-        cur_ver=nil
-        puts "                       -> cur_ver:#{cur_ver} and new_ver:#{new_ver}" if $DEBUG
-      end
-
-      # this need to conditionalize on apt-listbugs list
-      if new_ver.nil?
-        # 'apt-listbugs list' gives this status; I want to simulate installing the bug status of installing the current package anew
-        new_ver=cur_ver
-        cur_ver=nil
-      end
-      
-      # ignore bugs that have no fixed version info, and is closed with a XXXX-done
+      # ignore bugs that have no fixed version info, and are closed with a XXXX-done
       if fixed.nil? && bug_stat == "done"
-        puts "bug_number:##{bug_number} -- apparently closed with XXXX-done without version info" if $DEBUG
+        puts "bug apparently closed with XXXX-done without version info" if $DEBUG
         val = true
-        ##316121
-      else if new_ver.nil? 
-             # This is an impossible case
-             puts "ARRGH I shouldn't be here!" if $DEBUG
-             val=nil
+      else if new_ver.nil?
+             # no specific version was given for the package, which means that we are interested in all (non archived) bugs
+             puts "no package version given" if $DEBUG
+             val = false
            else if cur_ver.nil?
-                  # I don't have the current package installed, which means I want to check the new version
-                  val=true if ! am_i_buggy(new_ver, fixed, found)
+                  # no known installed version, which means that we want to check the new version
+                  val = true if ! am_i_buggy(new_ver, fixed, found)
                 else
-                  val=true if am_i_buggy(cur_ver, fixed, found) || ( ! am_i_buggy(new_ver, fixed, found))
+                  # both versions are known, which means that we want to check whether the upgrade may introduce this bug into the system
+                  val = true if am_i_buggy(cur_ver, fixed, found) || ( ! am_i_buggy(new_ver, fixed, found))
                 end
            end
       end
 
-      puts "in conclusion, For bug #{bug_number} comparing fixed: [#{fixed}], found:[#{found}], cur_ver:#{cur_ver} and new_ver:#{new_ver} results in removal:#{val} " if $DEBUG
+      puts "in conclusion, for bug #{bug_number} comparing fixed:[#{fixed}], found:[#{found}], cur_ver:#{cur_ver} and new_ver:#{new_ver} results in removal:#{val}" if $DEBUG
       val
     end
 
@@ -1058,7 +962,7 @@ module Factory
       step=(max/100)*10+1
       i=0
       yield _("Parsing Found/Fixed information..."), "0%"
-      
+
       bugs.delete_if { |bug|
 	val = false
 	name = bug.pkg_name
@@ -1066,13 +970,13 @@ module Factory
 	cur_ver = nil
 	new_ver = new_pkgs[name]["version"] if new_pkgs[name] != nil
 	cur_ver = cur_pkgs[name]["version"] if cur_pkgs[name] != nil
-        
+
         # show progress
-        yield _("Parsing Found/Fixed information..."), 
+        yield _("Parsing Found/Fixed information..."),
         "#{(i.to_f/max.to_f*100).to_i}%" if (i % step) == 0
         i += 1
 
-        val = true if bug_is_irrelevant(name, cur_ver, new_ver, 
+        val = true if bug_is_irrelevant(name, cur_ver, new_ver,
                                         bug.bug_number, bug.fixed, bug.found, bug.stat)
         val
       }
@@ -1096,16 +1000,17 @@ module Factory
 	val
       }
     end
-    
-    module_function :create, :delete_ignore_bugs,
+
+    module_function :delete_ignore_pkgs, :create, :delete_ignore_bugs,
     :delete_uninteresting_bugs,
     :delete_regexp_bugs,
     :bug_is_irrelevant,
     :am_i_buggy,
-    :delete_irrelevant_bugs, :delete_unwanted_tag_bugs, 
-    :find_max_version_below_ver, 
-    :find_min_version_above_ver, 
+    :delete_irrelevant_bugs, :delete_unwanted_tag_bugs,
+    :find_max_version_below_ver,
+    :find_min_version_above_ver,
     :iterate_fixed_found_version
+
   end
 
 end
@@ -1131,13 +1036,17 @@ class ConsoleFrontend
   def puts(msg)
     $stdout.puts msg
   end
-  
+
+
+  def tty
+    @tty ||= open("/dev/tty")
+  end
+
   def ask(msg)
     $stdout.print "#{msg} "
     $stdout.flush
     line = nil
-    @tty = open("/dev/tty") if @tty == nil
-    line = @tty.gets
+    line = self.tty.gets
     if line != nil
       line.chomp!
     end
@@ -1149,9 +1058,9 @@ class ConsoleFrontend
     while true
       msgyn = "#{msg}"
       if default == true
-	msgyn << "[Y/n]?"
+	msgyn << " [Y/n]"
       else
-	msgyn << "[y/N]?"
+	msgyn << " [y/N]"
       end
       a = ask msgyn
       if a == ""
